##################################
### WebSiteStatus.tcl          ###
### Version 1.0                ###
### By Wcc                     ###
### wcc@techmonkeys.org        ###
### http://www.dawgtcl.com:81/ ###
### EFnet #|DAWG|Tcl           ###
##################################

############################################################################
### Copyright  2000 - 2002 |DAWG| Scripting Group. All rights reserved. ###
############################################################################

###########################################################################
## This script checks if a web site is up or down. This is determined by ##
## wheather or not the web site sends a 200 (OK) numeric.                ##
###########################################################################

##############
## COMMANDS ##
###################################################################
## DCC ## .sitecheck <[on|off|url]> (Can be changed.)            ##
######### Checks the status of a url. "On" or "off" turn the pub ##
######### command on or off.                                     ##
###################################################################
## PUB ## !sitecheck <url> (Can be changed.)                     ##
######### Checks the status of a url.                            ##
###################################################################

##########################################################
## Just load the script, edit the settings, and rehash. ##
##########################################################

#############################
# Set the dcc command here. #
#############################

set websitestatus_setting(dcmd) "sitecheck"

#############################
# Set the pub command here. #
#############################

set websitestatus_setting(pcmd) "!sitecheck"

##########################################################################
# Set the flag required to use the pub and dcc commands here ("-" allows #
# all users).                                                            #
##########################################################################

set websitestatus_setting(flag) "-"

#####################################################
# Set the database filename here (used for on/off). #
#####################################################

set websitestatus_setting(db) "./scripts/websitestatus.db"

###################################
# Enable use of bold in DCC chat? #
###################################

set websitestatus_setting(bold) 1

#################################################
# Prefix "WEBSITESTATUS:" in DCC chat messages? #
#################################################

set websitestatus_setting(WEBSITESTATUS:) 1

####################
# Code begins here #
####################

if {![string match 1.6.* $version]} { putlog "\002WEBSITESTATUS:\002 \002WARNING:\002 This script is intended to run on eggdrop 1.6.x or later." }
if {[info tclversion] < 8.2} { putlog "\002WEBSITESTATUS:\002 \002WARNING:\002 This script is intended to run on Tcl Version 8.2 or later." }

package require http

bind dcc $websitestatus_setting(flag) $websitestatus_setting(dcmd) websitestatus_dcc
bind pub $websitestatus_setting(flag) $websitestatus_setting(pcmd) websitestatus_pub

proc websitestatus_readarray {array file} {
	upvar $array ours
	if {[catch {open $file r} fd]} { return }
	if {[array exists ours]} { unset ours }
	foreach line [split [read $fd] \n] {
		if {[string range $line 0 2] != "%!%" || [scan [string range $line 3 end] "%\[^!!!\]!!!%s" name data] <= 0} { continue }
		if {![info exists data]} { set data "" }
		set ours($name) $data
	}
	close $fd
}
proc websitestatus_savearray {array file} {
	upvar $array ours
	if {[catch {open $file w} fd] || ![array exists ours]} { return }
	foreach entry [array names ours] { puts $fd "%!%$entry!!!$ours($entry)" }
	close $fd
}
if {![array exists websitestatus_set] && [file exists $websitestatus_setting(db)]} { websitestatus_readarray websitestatus_set $websitestatus_setting(db) }
if {![info exists websitestatus_set(off)]} { set websitestatus_set(off) 0 }
proc websitestatus_dopre {} {
	if {!$::websitestatus_setting(WEBSITESTATUS:)} { return "" }
	if {!$::websitestatus_setting(bold)} { return "WEBSITESTATUS: " }
	return "\002WEBSITESTATUS:\002 "
}
proc websitestatus_notnumber {string} { return [regexp {[^0-9]} $string] }
proc websitestatus_dcc {hand idx text} {
	global websitestatus_set
	if {[string compare $text ""] == 0} { putdcc $idx "[websitestatus_dopre]Usage: $::websitestatus_setting(dcmd) <\[url|on|off\]>" }
	if {[string compare [string tolower [join [lindex [split $text] 0]]] "on"] == 0} {
		set websitestatus_set(off) 0
		putdcc $idx "[websitestatus_dopre]The $::websitestatus_setting(pcmd) is now enabled."
		websitestatus_savearray websitestatus_set $::websitestatus_setting(db); return
	}
	if {[string compare [string tolower [join [lindex [split $text] 0]]] "off"] == 0} {
		set websitestatus_set(off) 1
		putdcc $idx "[websitestatus_dopre]The $::websitestatus_setting(pcmd) is now disabled." ; return
		websitestatus_savearray websitestatus_set $::websitestatus_setting(db); return
	}
	websitestatus_go 0 $idx [join [lindex [split $text] 0]]
}
proc websitestatus_pub {nick uhost hand chan text} {
	if {$::websitestatus_set(off)} { return }
	if {[string compare $text ""] == 0} { putserv "PRIVMSG $chan :Usage: $::websitestatus_setting(pcmd) <url>" }
	websitestatus_go 1 $chan [join [lindex [split $text] 0]]
}
proc websitestatus_go {type dest url} {
	::http::config -useragent WebSiteStatus.tcl"
	if {[catch {set tok [::http::geturl $url]} error]} {
		putlog blah
		set text "The status of $url is: down."
		if {$type} { putserv "PRIVMSG $dest :$text" } { putdcc $dest "[websitestatus_dopre]$text" }
		return
	}
	if {[::http::ncode $tok] != 200} {
		set text "The status of $url is: down."
		if {$type} { putserv "PRIVMSG $dest :$text" } { putdcc $dest "[websitestatus_dopre]$text" }
		::http::cleanup $tok ; return
	}
	set text "The status of $url is: up."
	if {$type} { putserv "PRIVMSG $dest :$text" } { putdcc $dest "[websitestatus_dopre]$text" }
	::http::cleanup $tok
}

putlog "\002WEBSITESTATUS:\002 WebSiteStatus.tcl Version 1.0 by Wcc is loaded."
